@@ -3,7 +3,7 @@
 //
 // MIT License
 //
-// Copyright (c) 2014-2017 Michael Truog <mjtruog at protonmail dot com>
+// Copyright (c) 2014-2021 Michael Truog <mjtruog at protonmail dot com>
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -30,7 +30,16 @@ namespace CloudI;
 for ($i = 0; $i < ob_get_level(); $i++) { ob_end_flush(); }
 ob_implicit_flush(true);
 // an assert failure needs to cause php to exit
-assert_options(ASSERT_BAIL);
+if (! ini_get('assert.active'))
+    ini_set('assert.active', '1');
+if (! ini_get('assert.warning'))
+    ini_set('assert.warning', '1');
+if (! ini_get('assert.bail'))
+    ini_set('assert.bail', '1');
+if (ini_get('assert.quiet_eval'))
+    ini_set('assert.quiet_eval', '0');
+if (ini_get('assert.callback'))
+    ini_set('assert.callback', null);
 
 require dirname(__FILE__) . '/Erlang.php';
 
@@ -72,7 +81,10 @@ class API
     {
         $protocol_str = getenv('CLOUDI_API_INIT_PROTOCOL');
         if ($protocol_str == false)
+        {
+            fwrite(STDERR, "CloudI service execution must occur in CloudI\n");
             throw new InvalidInputException();
+        }
         $buffer_size_str = getenv('CLOUDI_API_INIT_BUFFER_SIZE');
         if ($buffer_size_str == false)
             throw new InvalidInputException();
@@ -89,7 +101,7 @@ class API
         $this->terminate = false;
         $this->size = intval($buffer_size_str);
         $this->callbacks = array();
-        $this->timeout_terminate = 1000; // TIMEOUT_TERMINATE_MIN
+        $this->timeout_terminate = 10; // TIMEOUT_TERMINATE_MIN
         $this->send(\Erlang\term_to_binary(new \Erlang\OtpErlangAtom('init')));
         list($this->process_index,
              $this->process_count,
@@ -330,6 +342,11 @@ class API
         return $this->timeout_terminate;
     }
 
+    public function priority_default()
+    {
+        return $this->priority_default;
+    }
+
     private function null_response($request_type, $name, $pattern,
                                    $request_info, $request,
                                    $timeout, $priority, $trans_id, $pid)
@@ -351,6 +368,7 @@ class API
             $function = array_shift($function_queue);
             $function_queue[] = $function;
         }
+        $return_null_response = false;
         switch ($command)
         {
             case MESSAGE_SEND_ASYNC:
@@ -376,41 +394,47 @@ class API
                     if (! is_string($response))
                         $response = '';
                 }
-                catch (InvalidInputException $e)
-                {
-                    throw $e;
-                }
                 catch (MessageDecodingException $e)
                 {
-                    throw $e;
+                    $this->terminate = true;
+                    $return_null_response = true;
                 }
                 catch (TerminateException $e)
                 {
-                    throw $e;
+                    $return_null_response = true;
                 }
                 catch (ReturnAsyncException $e)
                 {
                     return;
                 }
+                catch (ReturnSyncException $e)
+                {
+                    $this->terminate = true;
+                    fwrite(STDERR, "{$e->getMessage()}\n{$e}\n");
+                    return;
+                }
                 catch (ForwardAsyncException $e)
                 {
                     return;
                 }
-                catch (ReturnSyncException $e)
-                {
-                    echo "{$e->getMessage()}\n{$e}\n";
-                    assert(false);
-                    return;
-                }
                 catch (ForwardSyncException $e)
                 {
-                    echo "{$e->getMessage()}\n{$e}\n";
-                    assert(false);
+                    $this->terminate = true;
+                    fwrite(STDERR, "{$e->getMessage()}\n{$e}\n");
                     return;
                 }
-                catch (\Exception $e)
+                catch (\Error $e)
+                {
+                    fwrite(STDERR, "{$e->getMessage()}\n{$e}\n");
+                    exit(1);
+                }
+                catch (\Throwable $e)
+                {
+                    $return_null_response = true;
+                    fwrite(STDERR, "{$e->getMessage()}\n{$e}\n");
+                }
+                if ($return_null_response)
                 {
-                    echo "{$e->getMessage()}\n{$e}\n";
                     $response_info = '';
                     $response = '';
                 }
@@ -447,41 +471,47 @@ class API
                     if (! is_string($response))
                         $response = '';
                 }
-                catch (InvalidInputException $e)
-                {
-                    throw $e;
-                }
                 catch (MessageDecodingException $e)
                 {
-                    throw $e;
+                    $this->terminate = true;
+                    $return_null_response = true;
                 }
                 catch (TerminateException $e)
                 {
-                    throw $e;
+                    $return_null_response = true;
                 }
                 catch (ReturnSyncException $e)
                 {
                     return;
                 }
+                catch (ReturnAsyncException $e)
+                {
+                    $this->terminate = true;
+                    fwrite(STDERR, "{$e->getMessage()}\n{$e}\n");
+                    return;
+                }
                 catch (ForwardSyncException $e)
                 {
                     return;
                 }
-                catch (ReturnAsyncException $e)
-                {
-                    echo "{$e->getMessage()}\n{$e}\n";
-                    assert(false);
-                    return;
-                }
                 catch (ForwardAsyncException $e)
                 {
-                    echo "{$e->getMessage()}\n{$e}\n";
-                    assert(false);
+                    $this->terminate = true;
+                    fwrite(STDERR, "{$e->getMessage()}\n{$e}\n");
                     return;
                 }
-                catch (\Exception $e)
+                catch (\Error $e)
+                {
+                    fwrite(STDERR, "{$e->getMessage()}\n{$e}\n");
+                    exit(1);
+                }
+                catch (\Throwable $e)
+                {
+                    $return_null_response = true;
+                    fwrite(STDERR, "{$e->getMessage()}\n{$e}\n");
+                }
+                if ($return_null_response)
                 {
-                    echo "{$e->getMessage()}\n{$e}\n";
                     $response_info = '';
                     $response = '';
                 }
@@ -554,7 +584,10 @@ class API
     {
         if ($this->terminate)
         {
-            return false;
+            if ($external)
+                return false;
+            else
+                throw new TerminateException($this->timeout_terminate);
         }
         elseif ($external && ! $this->initialization_complete)
         {
@@ -676,6 +709,10 @@ class API
                                     $request_info, $request,
                                     $request_timeout, $priority, $trans_id,
                                     \Erlang\binary_to_term($pid));
+                    if ($this->terminate)
+                    {
+                        return false;
+                    }
                     break;
                 case MESSAGE_RECV_ASYNC:
                 case MESSAGE_RETURN_SYNC:
@@ -825,36 +862,71 @@ class API
         return $this->poll_request($timeout, true);
     }
 
-    private function text_key_value_parse($text)
+    public function shutdown($reason = null)
     {
-        $result = array();
-        $data = explode("\0", $text);
-        $size = count($data);
+        if (is_null($reason))
+            $reason = "";
+        $this->send(\Erlang\term_to_binary(
+            array(new \Erlang\OtpErlangAtom('shutdown'), $reason)));
+    }
+
+    private static function text_pairs_parse($text)
+    {
+        $pairs = array();
+        $text_segments = explode("\0", $text);
+        $size = count($text_segments);
         if ($size >= 2)
         {
             foreach (range(0, ($size - $size % 2) - 2, 2) as $i)
             {
-                $key = $data[$i];
-                if (isset($result[$key]))
+                $key = $text_segments[$i];
+                if (isset($pairs[$key]))
                 {
-                    $value = $result[$key];
+                    $value = $pairs[$key];
                     if (is_array($value))
-                        $value[] = $data[$i + 1];
+                        $value[] = $text_segments[$i + 1];
                     else
-                        $result[$key] = array($value, $data[$i + 1]);
+                        $pairs[$key] = array($value, $text_segments[$i + 1]);
                 }
                 else
                 {
-                    $result[$key] = $data[$i + 1];
+                    $pairs[$key] = $text_segments[$i + 1];
                 }
             }
         }
-        return $result;
+        return $pairs;
     }
 
-    public function info_key_value_parse($message_info)
+    private static function text_pairs_new($pairs, $response)
     {
-        return $this->text_key_value_parse($message_info);
+        $text = '';
+        foreach ($pairs as $key => $values)
+        {
+            if (is_string($values))
+            {
+                $text .= $key . "\0" . $values . "\0";
+            }
+            else
+            {
+                foreach ($values as $value)
+                {
+                    $text .= $key . "\0" . $value . "\0";
+                }
+            }
+        }
+        if ($response && empty($text))
+            $text = "\0";
+        return $text;
+    }
+
+    public static function info_key_value_parse($info)
+    {
+        return self::text_pairs_parse($info);
+    }
+
+    public static function info_key_value_new($pairs, $response = true)
+    {
+        return self::text_pairs_new($pairs, $response);
     }
 
     private function send($data)
@@ -973,4 +1045,12 @@ class TerminateException extends \Exception
     }
 }
 
+class FatalError extends \Error
+{
+    public function __construct($message)
+    {
+        parent::__construct($message, 0, null);
+    }
+}
+
 ?>
