@@ -1,5 +1,11 @@
+%% @doc Binds placeholder parameters to prepared statement, creating a "portal"
+%%
+%% ```
 %% > Bind
 %% < BindComplete
+%% '''
+%% @see cloudi_x_epgsql_cmd_parse
+%% @see cloudi_x_epgsql_cmd_execute
 -module(cloudi_x_epgsql_cmd_bind).
 -behaviour(cloudi_x_epgsql_command).
 -export([init/1, execute/2, handle_message/4]).
@@ -24,13 +30,11 @@ execute(Sock, #bind{stmt = Stmt, portal = PortalName, params = Params} = St) ->
     TypedParams = lists:zip(Types, Params),
     Bin1 = cloudi_x_epgsql_wire:encode_parameters(TypedParams, Codec),
     Bin2 = cloudi_x_epgsql_wire:encode_formats(Columns),
-    cloudi_x_epgsql_sock:send_multi(
-      Sock,
-      [
-       {?BIND, [PortalName, 0, StatementName, 0, Bin1, Bin2]},
-       {?FLUSH, []}
-      ]),
-    {ok, Sock, St}.
+    Commands = [
+       cloudi_x_epgsql_wire:encode_bind(PortalName, StatementName, Bin1, Bin2),
+       cloudi_x_epgsql_wire:encode_flush()
+      ],
+    {send_multi, Commands, Sock, St}.
 
 handle_message(?BIND_COMPLETE, <<>>, Sock, _State) ->
     {finish, ok, ok, Sock};
