@@ -3,7 +3,7 @@
 //
 // MIT License
 //
-// Copyright (c) 2011-2017 Michael Truog <mjtruog at gmail dot com>
+// Copyright (c) 2011-2021 Michael Truog <mjtruog at protonmail dot com>
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -44,6 +44,7 @@
 #include "cloudi_os_spawn.hpp"
 #include "cloudi_os_rlimit.hpp"
 #include "cloudi_os_owner.hpp"
+#include "cloudi_os_syscall_lock.hpp"
 #include "assert.hpp"
 
 namespace
@@ -89,7 +90,7 @@ namespace
             connect_EACCES,
             connect_EPERM,
             connect_EADDRINUSE,
-            connect_EAFNOSUPPORT,
+            connect_ENOENT,
             connect_EAGAIN,
             connect_EALREADY,
             connect_EBADF,                 // 50
@@ -200,8 +201,8 @@ namespace
                     return "connect_EPERM";
                 case connect_EADDRINUSE:
                     return "connect_EADDRINUSE";
-                case connect_EAFNOSUPPORT:
-                    return "connect_EAFNOSUPPORT";
+                case connect_ENOENT:
+                    return "connect_ENOENT";
                 case connect_EAGAIN:
                     return "connect_EAGAIN";
                 case connect_EALREADY:
@@ -372,56 +373,18 @@ namespace
                     return "SIGTRAP";
                 case 134:
                     return "SIGABRT";
-                case 135:
-                    return "SIGBUS/SIGEMT";
                 case 136:
                     return "SIGFPE";
                 case 137:
                     return "SIGKILL";
-                case 138:
-                    return "SIGUSR1/SIGBUS";
                 case 139:
                     return "SIGSEGV";
-                case 140:
-                    return "SIGUSR2/SIGSYS";
                 case 141:
                     return "SIGPIPE";
                 case 142:
                     return "SIGALRM";
                 case 143:
                     return "SIGTERM";
-                case 144:
-                    return "SIGUSR1/SIGURG";
-                case 145:
-                    return "SIGUSR2/SIGCHLD/SIGSTOP";
-                case 146:
-                    return "SIGCHLD/SIGCONT/SIGTSTP";
-                case 147:
-                    return "SIGCONT/SIGSTOP/SIGPWR";
-                case 148:
-                    return "SIGCHLD/SIGTSTP/SIGWINCH";
-                case 149:
-                    return "SIGTTIN/SIGURG";
-                case 150:
-                    return "SIGTTOU/SIGIO";
-                case 151:
-                    return "SIGSTOP/SIGURG/SIGIO";
-                case 152:
-                    return "SIGTSTP/SIGXCPU";
-                case 153:
-                    return "SIGCONT/SIGXFSZ";
-                case 154:
-                    return "SIGTTIN/SIGVTALRM";
-                case 155:
-                    return "SIGTTOU/SIGPROF";
-                case 156:
-                    return "SIGVTALRM/SIGWINCH";
-                case 157:
-                    return "SIGPROF/SIGIO/SIGPWR";
-                case 158:
-                    return "SIGUSR1/SIGXCPU/SIGPWR";
-                case 159:
-                    return "SIGUSR2/SIGXFSZ/SIGSYS";
                 default:
                     return 0;
             }
@@ -526,8 +489,8 @@ namespace
                     return connect_EPERM;
                 case EADDRINUSE:
                     return connect_EADDRINUSE;
-                case EAFNOSUPPORT:
-                    return connect_EAFNOSUPPORT;
+                case ENOENT:
+                    return connect_ENOENT;
                 case EAGAIN:
                     return connect_EAGAIN;
                 case EALREADY:
@@ -626,6 +589,40 @@ namespace
                     return GEPD::ExitStatus::write_unknown;
             }
         }
+
+    } // namespace spawn_status
+
+    uint32_t strings_count(char const * const p, uint32_t const len)
+    {
+        uint32_t count = 1;
+        assert(p[len - 1] == '\0');
+        if (len > 1)
+        {
+            ++count;
+            for (uint32_t i = 0; i < len - 1; ++i)
+            {
+                if (p[i] == '\0')
+                    ++count;
+            }
+        }
+        return count;
+    }
+
+    void strings_set(char ** const strings, uint32_t const count,
+                     char * const p, uint32_t const len)
+    {
+        uint32_t index = 0;
+        if (count > 1)
+        {
+            strings[index++] = p;
+            for (uint32_t i = 0; i < len - 1; ++i)
+            {
+                if (p[i] == '\0')
+                    strings[index++] = &(p[i + 1]);
+            }
+        }
+        strings[index++] = 0;
+        assert(index == count);
     }
 
     class process_data
@@ -640,7 +637,8 @@ namespace
                 m_index_stream1(0),
                 m_index_stream2(0),
                 m_stream1(1, 16384),
-                m_stream2(1, 16384)
+                m_stream2(1, 16384),
+                m_killed(false)
             {
             }
 
@@ -648,33 +646,39 @@ namespace
             {
                 GEPD::nfds -= 2;
 
-                // kills the pid if it isn't dead,
-                // to avoid blocking on a closed pipe
-                ::kill(m_pid, 9);
-                int status;
-                int const pid = ::waitpid(m_pid, &status, 0);
-                assert(pid == static_cast<signed>(m_pid));
+                if (m_killed == false)
+                {
+                    // after the stdout/stderr pipes have closed,
+                    // block execution here waiting for m_pid to terminate
+                    // (depend on cloudi_core_i_services_monitor killing
+                    //  the pid with SIGKILL after the termination timeout)
+                    int status;
+                    int const pid = ::waitpid(m_pid, &status, 0);
+                    assert(pid == static_cast<signed>(m_pid));
 
-                if (WIFEXITED(status))
-                {
-                    status = WEXITSTATUS(status);
-                }
-                else if (WIFSIGNALED(status))
-                {
-                    status = WTERMSIG(status) + 128;
-                }
-                else
-                {
-                    assert(false);
-                }
-                if (status != 0)
-                {
-                    char const * const error = spawn_status::string(status);
-                    std::cerr << "OS pid " << m_pid << " exited with ";
-                    if (error)
-                        std::cerr << error << std::endl;
+                    if (WIFEXITED(status))
+                    {
+                        status = WEXITSTATUS(status);
+                    }
+                    else if (WIFSIGNALED(status))
+                    {
+                        status = WTERMSIG(status) + 128;
+                    }
                     else
-                        std::cerr << status << std::endl;
+                    {
+                        assert(false);
+                    }
+                    if (status != 0)
+                    {
+                        char const * const error = spawn_status::string(status);
+                        std::cerr << "OS pid " << m_pid << " exited with ";
+                        if (error)
+                            std::cerr << error << std::endl;
+                        else if (status > 128)
+                            std::cerr << "SIG#" << (status - 128) << std::endl;
+                        else
+                            std::cerr << status << std::endl;
+                    }
                 }
             }
 
@@ -684,6 +688,14 @@ namespace
                 ::close(GEPD::fds[m_index_stderr].fd);
             }
 
+            void kill()
+            {
+                close();
+                assert(SIGKILL == 9);
+                ::kill(m_pid, SIGKILL);
+                m_killed = true;
+            }
+
             void shift()
             {
                 int const old_index_stdout = m_index_stdout;
@@ -752,9 +764,16 @@ namespace
             size_t m_index_stream2;
             realloc_ptr<unsigned char> m_stream1;
             realloc_ptr<unsigned char> m_stream2;
+            bool m_killed;
     };
 
     std::vector< copy_ptr<process_data> > processes;
+
+} // anonymous namespace
+
+bool terminate_now()
+{
+    return processes.empty();
 }
 
 int32_t spawn(char protocol,
@@ -767,6 +786,7 @@ int32_t spawn(char protocol,
               char * group_str, uint32_t group_str_len,
               int32_t nice,
               char * chroot_directory, uint32_t chroot_directory_len,
+              char * syscall_lock, uint32_t syscall_lock_len,
               char * directory, uint32_t directory_len,
               char * filename, uint32_t /*filename_len*/,
               char * argv, uint32_t argv_len,
@@ -886,13 +906,17 @@ int32_t spawn(char protocol,
             else if (domain == PF_LOCAL)
             {
                 char port_str[16];
-                ::snprintf(port_str, sizeof(port_str), "%d", ports[i]);
-                assert(socket_path_len <= 104 - 10);
+                int const port_str_len = ::snprintf(port_str, sizeof(port_str),
+                                                    "%d", ports[i]);
+                assert(socket_path_len <=
+                       static_cast<uint32_t>(104 - port_str_len));
                 struct sockaddr_un local;
+                ::memset(&local, 0, sizeof(local));
                 local.sun_family = domain;
-                ::memcpy(local.sun_path, socket_path, socket_path_len);
+                ::memcpy(local.sun_path, socket_path, socket_path_len - 1);
                 ::memcpy(&(local.sun_path[socket_path_len - 1]),
-                         port_str, sizeof(port_str));
+                         port_str, port_str_len);
+
                 if (::connect(sockfd,
                               reinterpret_cast<struct sockaddr *>(&local),
                               sizeof(local)) == -1)
@@ -912,64 +936,14 @@ int32_t spawn(char protocol,
             }
         }
 
-        int argv_count = 2;
-        {
-            assert(argv[argv_len - 1] == '\0');
-            for (size_t i = 1; i < argv_len; ++i)
-            {
-                if (argv[i] == '\0')
-                    ++argv_count;
-            }
-        }
-        char * execve_argv[argv_count];
-        {
-            int index = 0;
-            execve_argv[index++] = filename;
-            if (argv_count == 2)
-            {
-                execve_argv[index++] = 0;
-            }
-            else
-            {
-                execve_argv[index++] = argv;
-                for (size_t i = 0; i < argv_len - 1; ++i)
-                {
-                    if (argv[i] == '\0')
-                        execve_argv[index++] = &(argv[i + 1]);
-                }
-                execve_argv[index++] = 0;
-                assert(index == argv_count);
-            }
-        }
+        uint32_t const argv_count = 1 + strings_count(argv, argv_len);
+        char ** const execve_argv = new char*[argv_count];
+        execve_argv[0] = filename;
+        strings_set(&execve_argv[1], argv_count - 1, argv, argv_len);
 
-        int env_count = 1;
-        {
-            assert(env[env_len - 1] == '\0');
-            for (size_t i = 1; i < env_len; ++i)
-            {
-                if (env[i] == '\0')
-                    ++env_count;
-            }
-        }
-        char * execve_env[env_count];
-        {
-            if (env_count == 1)
-            {
-                execve_env[0] = 0;
-            }
-            else
-            {
-                int index = 0;
-                execve_env[index++] = env;
-                for (size_t i = 0; i < env_len - 1; ++i)
-                {
-                    if (env[i] == '\0')
-                        execve_env[index++] = &(env[i + 1]);
-                }
-                execve_env[index++] = 0;
-                assert(index == env_count);
-            }
-        }
+        uint32_t const env_count = strings_count(env, env_len);
+        char ** const execve_env = new char*[env_count];
+        strings_set(execve_env, env_count, env, env_len);
 
         if (owner_get(user_i, user_str, user_str_len,
                       group_i, group_str, group_str_len))
@@ -1005,6 +979,17 @@ int32_t spawn(char protocol,
             if (::chdir(directory))
                 ::_exit(spawn_status::invalid_input);
         }
+        if (syscall_lock_len > 1)
+        {
+            uint32_t const syscall_lock_count = strings_count(syscall_lock,
+                                                              syscall_lock_len);
+            char ** const syscall_names = new char*[syscall_lock_count];
+            strings_set(syscall_names, syscall_lock_count,
+                        syscall_lock, syscall_lock_len);
+            if (syscall_lock_set(syscall_names))
+                ::_exit(spawn_status::invalid_input);
+            delete [] syscall_names;
+        }
 
         ::execve(filename, execve_argv, execve_env);
         ::_exit(spawn_status::errno_exec());
@@ -1036,14 +1021,13 @@ int32_t spawn(char protocol,
     return pid;
 }
 
-
 int main()
 {
+    assert_initialize();
     ::signal(SIGPIPE, SIG_IGN); // write to a broken socket error
     typedef std::vector< copy_ptr<process_data> >::iterator iterator;
     assert(spawn_status::last_value == GEPD::ExitStatus::min);
 
-    int const timeout = -1; // milliseconds
     realloc_ptr<unsigned char> erlang_buffer(32768, 4194304); // 4MB
     realloc_ptr<unsigned char> stream1(1, 16384);
     realloc_ptr<unsigned char> stream2(1, 16384);
@@ -1051,7 +1035,7 @@ int main()
     if ((status = GEPD::init()))
         return status;
     int count;
-    while ((status = GEPD::wait(count, timeout, erlang_buffer,
+    while ((status = GEPD::wait(count, erlang_buffer,
                                 stream1, stream2)) == GEPD::ExitStatus::ready)
     {
         iterator itr = processes.begin();
@@ -1075,6 +1059,12 @@ int main()
             }
         }
     }
+    // kill all remaining processes with SIGKILL
+    for (iterator itr = processes.begin(); itr != processes.end(); ++itr)
+    {
+        (*itr)->kill();
+    }
+    processes.clear();
     return status;
 }
 
