@@ -8,7 +8,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2014-2018 Michael Truog <mjtruog at protonmail dot com>
+%%% Copyright (c) 2014-2021 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -29,8 +29,8 @@
 %%% DEALINGS IN THE SOFTWARE.
 %%%
 %%% @author Michael Truog <mjtruog at protonmail dot com>
-%%% @copyright 2014-2018 Michael Truog
-%%% @version 1.7.4 {@date} {@time}
+%%% @copyright 2014-2021 Michael Truog
+%%% @version 2.0.2 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_environment).
@@ -46,7 +46,7 @@
 -export_type([lookup/0]).
 
 -include("cloudi_environment.hrl").
--include_lib("cloudi_core/include/cloudi_logger.hrl").
+-include("cloudi_logger.hrl").
 -include_lib("kernel/include/file.hrl").
 
 %%%------------------------------------------------------------------------
@@ -63,7 +63,9 @@
     lookup().
 
 lookup() ->
-    cloudi_core_i_spawn:environment_lookup().
+    cloudi_x_trie:new(lists:map(fun(Entry) ->
+        cloudi_string:splitl($=, Entry, input)
+    end, os:getenv())).
 
 %%-------------------------------------------------------------------------
 %% @doc
@@ -95,8 +97,9 @@ status() ->
      ApplicationVersions4} = lists:keytake(compiler, 1, ApplicationVersions3),
     {value, {cloudi_core, _, RuntimeCloudIVersion},
      _} = lists:keytake(cloudi_core, 1, ApplicationVersions4),
+    RuntimeErlangCompilation = erlang_compilation(),
     RuntimeMachineProcessors = erlang:system_info(schedulers),
-    status_static() ++
+    ?CODE_STATUS_STATIC ++
     [{build_erlang_erts_c_compiler_version, erts_c_compiler_version()},
      {install_erlang_erts_time, status_file_time(FileErts)},
      {install_erlang_kernel_time, status_file_time(FileKernel)},
@@ -109,6 +112,7 @@ status() ->
      {runtime_erlang_stdlib_version, RuntimeErlangStdlibVersion},
      {runtime_erlang_sasl_version, RuntimeErlangSaslVersion},
      {runtime_erlang_compiler_version, RuntimeErlangCompilerVersion},
+     {runtime_erlang_compilation, RuntimeErlangCompilation},
      {runtime_cloudi_version, RuntimeCloudIVersion},
      {runtime_machine_processors, RuntimeMachineProcessors}].
 
@@ -124,8 +128,7 @@ status() ->
     string().
 
 transform(String) ->
-    Lookup = lookup(),
-    cloudi_core_i_spawn:environment_transform(String, Lookup).
+    transform(String, lookup()).
 
 %%-------------------------------------------------------------------------
 %% @doc
@@ -140,7 +143,7 @@ transform(String) ->
     string().
 
 transform(String, Lookup) ->
-    cloudi_core_i_spawn:environment_transform(String, Lookup).
+    transform(String, [], Lookup).
 
 %%%------------------------------------------------------------------------
 %%% Private functions
@@ -151,7 +154,7 @@ status_file_time(FilePath) ->
         {ok, #file_info{mtime = MTime}} ->
             cloudi_timestamp:seconds_epoch_to_string(MTime);
         {error, Reason} ->
-            ?LOG_ERROR("filesystem error (~ts): ~p",
+            ?LOG_ERROR("filesystem error (~ts): ~tp",
                        [FilePath, Reason]),
             ""
     end.
@@ -169,10 +172,79 @@ erts_c_compiler_version() ->
                                            || I <- erlang:tuple_to_list(V)]),
             erlang:atom_to_list(Name) ++ [$  | Version];
         {Name, Version} ->
-            cloudi_string:format("~p ~p", [Name, Version]);
+            cloudi_string:format("~tp ~tp", [Name, Version]);
         Unexpected ->
-            ?LOG_ERROR("erlang:system_info(c_compiler_used) invalid: ~p",
+            ?LOG_ERROR("erlang:system_info(c_compiler_used) invalid: ~tp",
                        [Unexpected]),
             ""
     end.
 
+-ifdef(ERLANG_OTP_VERSION_24_FEATURES).
+erlang_compilation() ->
+    case erlang:system_info(emu_flavor) of
+        jit ->
+            "jit";
+        _ ->
+            "aot"
+    end.
+-else.
+erlang_compilation() ->
+    "aot".
+-endif.
+
+% transform a string using a lookup containing environment variables
+% (the loop doesn't have error conditions by design)
+transform([], Output, _) ->
+    lists:reverse(Output);
+
+transform([$\\, $$ | String], Output, Lookup) ->
+    transform(String, [$$ | Output], Lookup);
+
+transform([$$ | String], Output, Lookup) ->
+    case String of
+        [${ | Rest] ->
+            transform_delimiter(Rest, Output, [], Lookup);
+        _ ->
+            transform_bare(String, Output, [], Lookup)
+    end;
+
+transform([C | String], Output, Lookup) ->
+    transform(String, [C | Output], Lookup).
+
+transform_delimiter([], Output, _, _) ->
+    lists:reverse(Output);
+
+transform_delimiter([$} | String], Output, Key, Lookup) ->
+    transform_value(Key, String, Output, Lookup);
+
+transform_delimiter([$= | String], Output, _, Lookup) ->
+    transform(String, Output, Lookup);
+
+transform_delimiter([C | String], Output, Key, Lookup) ->
+    transform_delimiter(String, Output, [C | Key], Lookup).
+
+transform_bare([] = String, Output, Key, Lookup) ->
+    transform_value(Key, String, Output, Lookup);
+
+transform_bare([C | String], Output, _, Lookup)
+    when C == $}; C == $= ->
+    transform(String, Output, Lookup);
+
+transform_bare([C | String], Output, Key, Lookup)
+    when C /= $$, C /= $/, C /= $ , C /= $', C /= $", C /= $` ->
+    transform_bare(String, Output, [C | Key], Lookup);
+
+transform_bare(String, Output, Key, Lookup) ->
+    transform_value(Key, String, Output, Lookup).
+
+transform_value([], String, Output, Lookup) ->
+    transform(String, Output, Lookup);
+
+transform_value(Key, String, Output, Lookup) ->
+    case cloudi_x_trie:find(lists:reverse(Key), Lookup) of
+        {ok, Value} ->
+            transform(String, lists:reverse(Value, Output), Lookup);
+        error ->
+            transform(String, Output, Lookup)
+    end.
+
