@@ -7,7 +7,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2014-2018 Michael Truog <mjtruog at protonmail dot com>
+%%% Copyright (c) 2014-2020 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -28,8 +28,8 @@
 %%% DEALINGS IN THE SOFTWARE.
 %%%
 %%% @author Michael Truog <mjtruog at protonmail dot com>
-%%% @copyright 2014-2018 Michael Truog
-%%% @version 1.7.4 {@date} {@time}
+%%% @copyright 2014-2020 Michael Truog
+%%% @version 1.8.1 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_service_SUITE).
@@ -68,6 +68,7 @@
          t_service_internal_terminate_4/1,
          t_service_internal_update_1/1,
          t_service_internal_log_1/1,
+         t_service_internal_idle_1/1,
          t_cloudi_args_type_1/1,
          t_cloudi_service_name_1/1]).
 
@@ -96,6 +97,9 @@
 -include_lib("common_test/include/ct.hrl").
 -include_lib("cloudi_core/include/cloudi_logger.hrl").
 
+-ifndef(CLOUDI_TEST_TIMEOUT).
+-define(CLOUDI_TEST_TIMEOUT, 10). % seconds
+-endif.
 -define(VSN, {"test", "version", "(any erlang term data can be used)"}).
 -define(SERVICE_PREFIX1, "/").
 -define(SERVICE_SUFFIX1, "service_name").
@@ -155,6 +159,8 @@ cloudi_service_init(Args, ?SERVICE_PREFIX1, _Timeout, Dispatcher) ->
                                                       undefined, undefined),
             {ok, _, _} = cloudi_service:recv_async(Dispatcher, TransId),
             undefined;
+        Mode =:= idle ->
+            idle;
         Mode =:= terminate_sleep ->
             cloudi_service:subscribe(Dispatcher,
                                      ?SERVICE_SUFFIX1),
@@ -162,23 +168,23 @@ cloudi_service_init(Args, ?SERVICE_PREFIX1, _Timeout, Dispatcher) ->
     end,
     {ok, #state{mode = NewMode}}.
 
-cloudi_service_handle_request(Type, Name, Pattern,
+cloudi_service_handle_request(RequestType, Name, Pattern,
                               ?REQUEST_INFO1 = RequestInfo, ?REQUEST1 = Request,
                               Timeout, Priority, TransId, Pid,
                               #state{mode = reply,
                                      requests = Requests} = State,
                               _Dispatcher) ->
-    NewRequests = [{Type, Name, Pattern, RequestInfo, Request,
+    NewRequests = [{RequestType, Name, Pattern, RequestInfo, Request,
                     Timeout, Priority, TransId, Pid} | Requests],
     {reply, ?RESPONSE_INFO1, ?RESPONSE1,
      State#state{requests = NewRequests}};
-cloudi_service_handle_request(_Type, _Name, _Pattern,
+cloudi_service_handle_request(_RequestType, _Name, _Pattern,
                               ?REQUEST_INFO2, ?REQUEST2,
                               _Timeout, _Priority, _TransId, _Pid,
                               #state{requests = Requests} = State,
                               _Dispatcher) ->
     {reply, <<>>, lists:reverse(Requests), State#state{requests = []}};
-cloudi_service_handle_request(_Type, Name, _Pattern,
+cloudi_service_handle_request(_RequestType, Name, _Pattern,
                               ?REQUEST_INFO3, ?REQUEST3,
                               _Timeout, _Priority, _TransId, _Pid,
                               #state{mode = reply} = State,
@@ -188,13 +194,13 @@ cloudi_service_handle_request(_Type, Name, _Pattern,
                                                 undefined, undefined),
     {ok, Responses} = cloudi_service:recv_asyncs(Dispatcher, TransIds),
     {reply, Responses, State};
-cloudi_service_handle_request(_Type, _Name, _Pattern,
+cloudi_service_handle_request(_RequestType, _Name, _Pattern,
                               ?REQUEST_INFO4, ?REQUEST4,
                               _Timeout, _Priority, _TransId, _Pid,
                               #state{} = State,
                               _Dispatcher) ->
     {reply, ?RESPONSE_INFO1, ?RESPONSE1, State};
-cloudi_service_handle_request(_Type, _Name, _Pattern,
+cloudi_service_handle_request(_RequestType, _Name, _Pattern,
                               ?REQUEST_INFO5, ?REQUEST5,
                               _Timeout, _Priority, _TransId, _Pid,
                               #state{mode = reply,
@@ -202,13 +208,13 @@ cloudi_service_handle_request(_Type, _Name, _Pattern,
                               _Dispatcher) ->
     {reply, ?RESPONSE_INFO1, ?RESPONSE1,
      State#state{count = Count + 1}};
-cloudi_service_handle_request(_Type, _Name, _Pattern,
+cloudi_service_handle_request(_RequestType, _Name, _Pattern,
                               ?REQUEST_INFO6, ?REQUEST6,
                               _Timeout, _Priority, _TransId, _Pid,
                               #state{count = Count} = State,
                               _Dispatcher) ->
     {reply, Count, State};
-cloudi_service_handle_request(_Type, _Name, _Pattern,
+cloudi_service_handle_request(_RequestType, _Name, _Pattern,
                               ?REQUEST_INFO7, ?REQUEST7,
                               _Timeout, _Priority, _TransId, _Pid,
                               #state{} = State,
@@ -247,6 +253,10 @@ cloudi_service_terminate(_Reason, _Timeout,
 cloudi_service_terminate(_Reason, _Timeout,
                          #state{count = 60}) ->
     % t_service_internal_aspects_1/1 result
+    ok;
+cloudi_service_terminate(_Reason, _Timeout,
+                         #state{count = 100}) ->
+    % t_service_internal_update_1/1 result
     ok.
 
 %%%------------------------------------------------------------------------
@@ -274,14 +284,15 @@ groups() ->
        t_service_internal_terminate_3,
        t_service_internal_terminate_4,
        t_service_internal_update_1,
-       t_service_internal_log_1]},
+       t_service_internal_log_1,
+       t_service_internal_idle_1]},
      {cloudi_modules_1, [parallel],
       [t_cloudi_args_type_1,
        t_cloudi_service_name_1]}].
 
 suite() ->
     [{ct_hooks, [cth_surefire]},
-     {timetrap, 10100}].
+     {timetrap, {seconds, ?CLOUDI_TEST_TIMEOUT}}].
 
 init_per_suite(Config) ->
     ok = cloudi_x_reltool_util:application_start(sasl,
@@ -478,6 +489,18 @@ init_per_testcase(TestCase, Config)
            {duo_mode, true}]}]
         ], infinity),
     [{service_ids, ServiceIds} | Config];
+init_per_testcase(TestCase, Config)
+    when (TestCase =:= t_service_internal_idle_1) ->
+    init_per_testcase(TestCase),
+    {ok, ServiceIds} = cloudi_service_api:services_add([
+        % using proplist configuration format, not the tuple/record format
+        [{prefix, ?SERVICE_PREFIX1},
+         {module, ?MODULE},
+         {args, [{mode, idle}]},
+         {options,
+          [{automatic_loading, false}]}]
+        ], infinity),
+    [{service_ids, ServiceIds} | Config];
 init_per_testcase(TestCase, Config) ->
     init_per_testcase(TestCase),
     Config.
@@ -542,7 +565,8 @@ t_service_internal_sync_1(_Config) ->
                                    ServiceName,
                                    ?REQUEST2),
     true = cloudi_x_uuid:is_v1(TransId2),
-    true = (TransId2 > TransId1),
+    true = (cloudi_x_uuid:get_v1_time(TransId2) >
+            cloudi_x_uuid:get_v1_time(TransId1)),
     {error,
      {service_internal_start_failed,
       {error, invalid_state}}} = cloudi_service_api:services_add([
@@ -627,7 +651,8 @@ t_service_internal_async_1(_Config) ->
                                    ?REQUEST2),
     true = (Timeout2 > (TimeoutMax - 1000)) andalso (Timeout2 =< TimeoutMax),
     true = cloudi_x_uuid:is_v1(TransId2),
-    true = (TransId2 > TransId1),
+    true = (cloudi_x_uuid:get_v1_time(TransId2) >
+            cloudi_x_uuid:get_v1_time(TransId1)),
     {error,
      {service_internal_start_failed,
       {error, invalid_state}}} = cloudi_service_api:services_add([
@@ -661,9 +686,12 @@ t_service_internal_async_2(_Config) ->
      Context4} = cloudi:recv_async(Context3, <<0:128>>),
     {{ok, ?RESPONSE_INFO1, ?RESPONSE1, TransId4},
      Context5} = cloudi:recv_async(Context4, <<0:128>>),
-    true = (TransId1 < TransId2) andalso
-           (TransId2 < TransId3) andalso
-           (TransId3 < TransId4),
+    true = (cloudi_x_uuid:get_v1_time(TransId1) <
+            cloudi_x_uuid:get_v1_time(TransId2)) andalso
+           (cloudi_x_uuid:get_v1_time(TransId2) <
+            cloudi_x_uuid:get_v1_time(TransId3)) andalso
+           (cloudi_x_uuid:get_v1_time(TransId3) <
+            cloudi_x_uuid:get_v1_time(TransId4)),
     {{ok,
       [{'send_async', ServiceName, ServiceName, ?REQUEST_INFO1, ?REQUEST1,
         Timeout1, 0, TransId1, Self},
@@ -696,9 +724,12 @@ t_service_internal_async_2(_Config) ->
        {?RESPONSE_INFO1, ?RESPONSE1, TransId7},
        {?RESPONSE_INFO1, ?RESPONSE1, TransId8}]},
      Context8} = cloudi:recv_asyncs(Context7, TransIds),
-    true = (TransId5 < TransId6) andalso
-           (TransId6 < TransId7) andalso
-           (TransId7 < TransId8),
+    true = (cloudi_x_uuid:get_v1_time(TransId5) <
+            cloudi_x_uuid:get_v1_time(TransId6)) andalso
+           (cloudi_x_uuid:get_v1_time(TransId6) <
+            cloudi_x_uuid:get_v1_time(TransId7)) andalso
+           (cloudi_x_uuid:get_v1_time(TransId7) <
+            cloudi_x_uuid:get_v1_time(TransId8)),
     {{ok,
       [{'send_async', ServiceName, ServiceName, ?REQUEST_INFO1, ?REQUEST1,
         Timeout5, 0, TransId5, Self},
@@ -736,9 +767,12 @@ t_service_internal_async_3(_Config) ->
        {?RESPONSE_INFO1, ?RESPONSE1, TransId3},
        {?RESPONSE_INFO1, ?RESPONSE1, TransId4}]},
      _Context1} = cloudi:send_sync(Context0, ServiceName, ?REQUEST3),
-    true = (TransId1 < TransId2) andalso
-           (TransId2 < TransId3) andalso
-           (TransId3 < TransId4),
+    true = (cloudi_x_uuid:get_v1_time(TransId1) <
+            cloudi_x_uuid:get_v1_time(TransId2)) andalso
+           (cloudi_x_uuid:get_v1_time(TransId2) <
+            cloudi_x_uuid:get_v1_time(TransId3)) andalso
+           (cloudi_x_uuid:get_v1_time(TransId3) <
+            cloudi_x_uuid:get_v1_time(TransId4)),
     true = cloudi_x_uuid:is_v1(TransId1),
     true = cloudi_x_uuid:is_v1(TransId2),
     true = cloudi_x_uuid:is_v1(TransId3),
@@ -905,6 +939,14 @@ t_service_internal_log_1(_Config) ->
     ?LOG_INFO("Logging metadata", []),
     ok.
 
+t_service_internal_idle_1(Config) ->
+    [ServiceId] = ?config(service_ids, Config),
+    {ok, []} = cloudi_service_api:
+               service_subscriptions(ServiceId, infinity),
+    {error, not_found} = cloudi_service_api:
+                         service_subscriptions(<<0:128>>, infinity),
+    ok.
+
 t_cloudi_args_type_1(_Config) ->
     % based on cloudi_service_name:suffix/2 but enforcing checks on whether
     % it is a service name or service name pattern for service initialization
