@@ -7,7 +7,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2014-2020 Michael Truog <mjtruog at protonmail dot com>
+%%% Copyright (c) 2014-2021 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -28,8 +28,8 @@
 %%% DEALINGS IN THE SOFTWARE.
 %%%
 %%% @author Michael Truog <mjtruog at protonmail dot com>
-%%% @copyright 2014-2020 Michael Truog
-%%% @version 1.8.1 {@date} {@time}
+%%% @copyright 2014-2021 Michael Truog
+%%% @version 2.0.2 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_service_filesystem_SUITE).
@@ -511,9 +511,9 @@ t_filesystem_basic_read_wcache_1(_Config) ->
       [{<<"content-type">>, <<"application/octet-stream">>},
        {<<"content-disposition">>,
         <<"attachment; filename=\"ASCII.bin\"">>},
-       {<<"etag">>, ETag},
-       {<<"cache-control">>, <<"public,max-age=1">>},
+       {<<"cache-control">>, <<"public">>},
        {<<"expires">>, _},
+       {<<"etag">>, ETag},
        {<<"last-modified">>, LastModified},
        {<<"date">>, _},
        {<<"accept-ranges">>, <<"bytes">>}],
@@ -531,9 +531,9 @@ t_filesystem_basic_read_wcache_1(_Config) ->
     {{ok,
       [{<<"status">>, <<"206">>},
        {<<"content-type">>, <<"multipart/byteranges">>},
-       {<<"etag">>, ETag},
-       {<<"cache-control">>, <<"public,max-age=1">>},
+       {<<"cache-control">>, <<"public">>},
        {<<"expires">>, _},
+       {<<"etag">>, ETag},
        {<<"last-modified">>, LastModified},
        {<<"date">>, _},
        {<<"accept-ranges">>, <<"bytes">>}],
@@ -546,9 +546,9 @@ t_filesystem_basic_read_wcache_1(_Config) ->
       [{<<"status">>, <<"206">>},
        {<<"content-type">>,
         <<"multipart/byteranges; boundary=", Boundary1:BSize/bitstring>>},
-       {<<"etag">>, ETag},
-       {<<"cache-control">>, <<"public,max-age=1">>},
+       {<<"cache-control">>, <<"public">>},
        {<<"expires">>, _},
+       {<<"etag">>, ETag},
        {<<"last-modified">>, LastModified},
        {<<"date">>, _},
        {<<"accept-ranges">>, <<"bytes">>}],
@@ -602,9 +602,9 @@ t_filesystem_basic_read_wcache_1(_Config) ->
       [{<<"status">>, <<"206">>},
        {<<"content-type">>,<<"application/octet-stream">>},
        {<<"content-range">>, <<"bytes 97-99/127">>},
-       {<<"etag">>, ETag},
-       {<<"cache-control">>, <<"public,max-age=1">>},
+       {<<"cache-control">>, <<"public">>},
        {<<"expires">>, _},
+       {<<"etag">>, ETag},
        {<<"last-modified">>, LastModified},
        {<<"date">>, _},
        {<<"accept-ranges">>, <<"bytes">>}],
@@ -621,9 +621,9 @@ t_filesystem_basic_read_wcache_1(_Config) ->
       [{<<"status">>, <<"206">>},
        {<<"content-type">>,<<"application/octet-stream">>},
        {<<"content-range">>, <<"bytes 97-99/127">>},
-       {<<"etag">>, ETag},
-       {<<"cache-control">>, <<"public,max-age=1">>},
+       {<<"cache-control">>, <<"public">>},
        {<<"expires">>, _},
+       {<<"etag">>, ETag},
        {<<"last-modified">>, LastModified},
        {<<"date">>, _},
        {<<"accept-ranges">>, <<"bytes">>}],
@@ -640,9 +640,9 @@ t_filesystem_basic_read_wcache_1(_Config) ->
       [{<<"content-type">>, <<"application/octet-stream">>},
        {<<"content-disposition">>,
         <<"attachment; filename=\"ASCII.bin\"">>},
-       {<<"etag">>, ETag},
-       {<<"cache-control">>, <<"public,max-age=1">>},
+       {<<"cache-control">>, <<"public">>},
        {<<"expires">>, _},
+       {<<"etag">>, ETag},
        {<<"last-modified">>, LastModified},
        {<<"date">>, _},
        {<<"accept-ranges">>, <<"bytes">>}],
@@ -675,9 +675,9 @@ t_filesystem_basic_read_wcache_1(_Config) ->
       [{<<"content-type">>, <<"application/octet-stream">>},
        {<<"content-disposition">>,
         <<"attachment; filename=\"ASCII.bin\"">>},
-       {<<"etag">>, ETag},
-       {<<"cache-control">>, <<"public,max-age=1">>},
+       {<<"cache-control">>, <<"public">>},
        {<<"expires">>, _},
+       {<<"etag">>, ETag},
        {<<"last-modified">>, LastModified},
        {<<"date">>, _},
        {<<"accept-ranges">>, <<"bytes">>}],
@@ -694,9 +694,9 @@ t_filesystem_basic_read_wcache_1(_Config) ->
       [{<<"status">>, <<"206">>},
        {<<"content-type">>,<<"application/octet-stream">>},
        {<<"content-range">>, <<"bytes 123-126/127">>},
-       {<<"etag">>, ETag},
-       {<<"cache-control">>, <<"public,max-age=1">>},
+       {<<"cache-control">>, <<"public">>},
        {<<"expires">>, _},
+       {<<"etag">>, ETag},
        {<<"last-modified">>, LastModified},
        {<<"date">>, _},
        {<<"accept-ranges">>, <<"bytes">>}],
@@ -775,9 +775,9 @@ t_filesystem_basic_write_truncate_wcache_1(Config) ->
       [{<<"content-type">>, <<"text/plain">>},
        {<<"content-disposition">>,
         <<"attachment; filename=\"" ?WRITABLE_FILENAME1 "\"">>},
-       {<<"etag">>, ETag0},
-       {<<"cache-control">>, <<"public,max-age=1">>},
+       {<<"cache-control">>, <<"public">>},
        {<<"expires">>, _},
+       {<<"etag">>, ETag0},
        {<<"last-modified">>, LastModified0},
        {<<"date">>, _},
        {<<"accept-ranges">>, <<"bytes">>}],
@@ -790,9 +790,9 @@ t_filesystem_basic_write_truncate_wcache_1(Config) ->
       [{<<"content-type">>, <<"text/plain">>},
        {<<"content-disposition">>,
         <<"attachment; filename=\"" ?WRITABLE_FILENAME1 "\"">>},
-       {<<"etag">>, ETag1},
-       {<<"cache-control">>, <<"public,max-age=1">>},
+       {<<"cache-control">>, <<"public">>},
        {<<"expires">>, _},
+       {<<"etag">>, ETag1},
        {<<"last-modified">>, LastModified1},
        {<<"date">>, _},
        {<<"accept-ranges">>, <<"bytes">>}],
@@ -805,9 +805,9 @@ t_filesystem_basic_write_truncate_wcache_1(Config) ->
       [{<<"content-type">>, <<"text/plain">>},
        {<<"content-disposition">>,
         <<"attachment; filename=\"" ?WRITABLE_FILENAME1 "\"">>},
-       {<<"etag">>, ETag2},
-       {<<"cache-control">>, <<"public,max-age=1">>},
+       {<<"cache-control">>, <<"public">>},
        {<<"expires">>, _},
+       {<<"etag">>, ETag2},
        {<<"last-modified">>, LastModified2},
        {<<"date">>, _},
        {<<"accept-ranges">>, <<"bytes">>}],
@@ -1001,9 +1001,9 @@ t_filesystem_basic_write_append_wcache_1(Config) ->
       [{<<"content-type">>, <<"text/plain">>},
        {<<"content-disposition">>,
         <<"attachment; filename=\"" ?WRITABLE_FILENAME1 "\"">>},
-       {<<"etag">>, ETag0},
-       {<<"cache-control">>, <<"public,max-age=1">>},
+       {<<"cache-control">>, <<"public">>},
        {<<"expires">>, _},
+       {<<"etag">>, ETag0},
        {<<"last-modified">>, LastModified0},
        {<<"date">>, _},
        {<<"accept-ranges">>, <<"bytes">>}],
@@ -1016,9 +1016,9 @@ t_filesystem_basic_write_append_wcache_1(Config) ->
       [{<<"content-type">>, <<"text/plain">>},
        {<<"content-disposition">>,
         <<"attachment; filename=\"" ?WRITABLE_FILENAME1 "\"">>},
-       {<<"etag">>, ETag1},
-       {<<"cache-control">>, <<"public,max-age=1">>},
+       {<<"cache-control">>, <<"public">>},
        {<<"expires">>, _},
+       {<<"etag">>, ETag1},
        {<<"last-modified">>, LastModified1},
        {<<"date">>, _},
        {<<"accept-ranges">>, <<"bytes">>}],
@@ -1036,9 +1036,9 @@ t_filesystem_basic_write_append_wcache_1(Config) ->
       [{<<"content-type">>, <<"text/plain">>},
        {<<"content-disposition">>,
         <<"attachment; filename=\"" ?WRITABLE_FILENAME1 "\"">>},
-       {<<"etag">>, ETag2},
-       {<<"cache-control">>, <<"public,max-age=1">>},
+       {<<"cache-control">>, <<"public">>},
        {<<"expires">>, _},
+       {<<"etag">>, ETag2},
        {<<"last-modified">>, LastModified2},
        {<<"date">>, _},
        {<<"accept-ranges">>, <<"bytes">>}],
@@ -1059,9 +1059,9 @@ t_filesystem_basic_write_append_wcache_1(Config) ->
       [{<<"content-type">>, <<"text/plain">>},
        {<<"content-disposition">>,
         <<"attachment; filename=\"" ?WRITABLE_FILENAME1 "\"">>},
-       {<<"etag">>, ETag3},
-       {<<"cache-control">>, <<"public,max-age=1">>},
+       {<<"cache-control">>, <<"public">>},
        {<<"expires">>, _},
+       {<<"etag">>, ETag3},
        {<<"last-modified">>, LastModified3},
        {<<"date">>, _},
        {<<"accept-ranges">>, <<"bytes">>}],
@@ -1080,9 +1080,9 @@ t_filesystem_basic_write_append_wcache_1(Config) ->
       [{<<"content-type">>, <<"text/plain">>},
        {<<"content-disposition">>,
         <<"attachment; filename=\"" ?WRITABLE_FILENAME1 "\"">>},
-       {<<"etag">>, ETag4},
-       {<<"cache-control">>, <<"public,max-age=1">>},
+       {<<"cache-control">>, <<"public">>},
        {<<"expires">>, _},
+       {<<"etag">>, ETag4},
        {<<"last-modified">>, LastModified4},
        {<<"date">>, _},
        {<<"accept-ranges">>, <<"bytes">>}],
@@ -1101,9 +1101,9 @@ t_filesystem_basic_write_append_wcache_1(Config) ->
       [{<<"content-type">>, <<"text/plain">>},
        {<<"content-disposition">>,
         <<"attachment; filename=\"" ?WRITABLE_FILENAME1 "\"">>},
-       {<<"etag">>, ETag5},
-       {<<"cache-control">>, <<"public,max-age=1">>},
+       {<<"cache-control">>, <<"public">>},
        {<<"expires">>, _},
+       {<<"etag">>, ETag5},
        {<<"last-modified">>, LastModified5},
        {<<"date">>, _},
        {<<"accept-ranges">>, <<"bytes">>}],
@@ -1134,9 +1134,9 @@ t_filesystem_basic_write_append_wcache_1(Config) ->
       [{<<"content-type">>, <<"text/plain">>},
        {<<"content-disposition">>,
         <<"attachment; filename=\"" ?WRITABLE_FILENAME1 "\"">>},
-       {<<"etag">>, ETag7},
-       {<<"cache-control">>, <<"public,max-age=1">>},
+       {<<"cache-control">>, <<"public">>},
        {<<"expires">>, _},
+       {<<"etag">>, ETag7},
        {<<"last-modified">>, LastModified7},
        {<<"date">>, _},
        {<<"accept-ranges">>, <<"bytes">>}],
@@ -1153,9 +1153,9 @@ t_filesystem_basic_write_append_wcache_1(Config) ->
       [{<<"content-type">>, <<"text/plain">>},
        {<<"content-disposition">>,
         <<"attachment; filename=\"" ?WRITABLE_FILENAME1 "\"">>},
-       {<<"etag">>, ETag8},
-       {<<"cache-control">>, <<"public,max-age=1">>},
+       {<<"cache-control">>, <<"public">>},
        {<<"expires">>, _},
+       {<<"etag">>, ETag8},
        {<<"last-modified">>, LastModified8},
        {<<"date">>, _},
        {<<"accept-ranges">>, <<"bytes">>}],
@@ -1175,14 +1175,13 @@ t_filesystem_basic_read_cache_1(_Config) ->
     Request = <<>>,
     Timeout = undefined, % default
     Priority = undefined, % default
-    RefreshString = erlang:integer_to_binary(?REFRESH),
     {{ok,
       [{<<"content-type">>, <<"application/octet-stream">>},
        {<<"content-disposition">>,
         <<"attachment; filename=\"ASCII.bin\"">>},
-       {<<"etag">>, ETag},
-       {<<"cache-control">>, <<"public,max-age=", RefreshString/binary>>},
+       {<<"cache-control">>, <<"public">>},
        {<<"expires">>, _},
+       {<<"etag">>, ETag},
        {<<"last-modified">>, LastModified},
        {<<"date">>, _},
        {<<"accept-ranges">>, <<"bytes">>}],
@@ -1206,9 +1205,9 @@ t_filesystem_basic_read_cache_1(_Config) ->
       [{<<"content-type">>, <<"application/octet-stream">>},
        {<<"content-disposition">>,
         <<"attachment; filename=\"ASCII.bin\"">>},
-       {<<"etag">>, ETag},
-       {<<"cache-control">>, <<"public,max-age=", RefreshString/binary>>},
+       {<<"cache-control">>, <<"public">>},
        {<<"expires">>, _},
+       {<<"etag">>, ETag},
        {<<"last-modified">>, LastModified},
        {<<"date">>, _},
        {<<"accept-ranges">>, <<"bytes">>}],
@@ -1231,9 +1230,9 @@ t_filesystem_basic_read_cache_1(_Config) ->
       [{<<"content-type">>, <<"application/octet-stream">>},
        {<<"content-disposition">>,
         <<"attachment; filename=\"ASCII.bin\"">>},
-       {<<"etag">>, ETag},
-       {<<"cache-control">>, <<"public,max-age=", RefreshString/binary>>},
+       {<<"cache-control">>, <<"public">>},
        {<<"expires">>, _},
+       {<<"etag">>, ETag},
        {<<"last-modified">>, LastModified},
        {<<"date">>, _},
        {<<"accept-ranges">>, <<"bytes">>}],
@@ -1257,9 +1256,9 @@ t_filesystem_basic_read_cache_1(_Config) ->
       [{<<"content-type">>, <<"application/octet-stream">>},
        {<<"content-disposition">>,
         <<"attachment; filename=\"ASCII.bin\"">>},
-       {<<"etag">>, ETag},
-       {<<"cache-control">>, <<"public,max-age=", RefreshString/binary>>},
+       {<<"cache-control">>, <<"public">>},
        {<<"expires">>, _},
+       {<<"etag">>, ETag},
        {<<"last-modified">>, LastModified},
        {<<"date">>, _},
        {<<"accept-ranges">>, <<"bytes">>}],
@@ -1282,9 +1281,9 @@ t_filesystem_basic_read_cache_1(_Config) ->
       [{<<"content-type">>, <<"application/octet-stream">>},
        {<<"content-disposition">>,
         <<"attachment; filename=\"ASCII.bin\"">>},
-       {<<"etag">>, ETag},
-       {<<"cache-control">>, <<"public,max-age=", RefreshString/binary>>},
+       {<<"cache-control">>, <<"public">>},
        {<<"expires">>, _},
+       {<<"etag">>, ETag},
        {<<"last-modified">>, LastModified},
        {<<"date">>, _},
        {<<"accept-ranges">>, <<"bytes">>}],
