@@ -30,7 +30,7 @@
 %%%
 %%% @author Michael Truog <mjtruog at protonmail dot com>
 %%% @copyright 2012-2021 Michael Truog
-%%% @version 2.0.2 {@date} {@time}
+%%% @version 2.0.3 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_http_cowboy_handler).
@@ -624,21 +624,18 @@ upgrade_to_websocket(Req,
             NameIncoming ++ "/get"
     end,
     PathRawStr = erlang:binary_to_list(PathRaw),
-    PeerShort = erlang:list_to_binary(inet_parse:ntoa(ClientIpAddr)),
-    PeerLong = cloudi_ip_address:to_binary(ClientIpAddr),
-    PeerPort = erlang:integer_to_binary(ClientPort),
+    SourceAddress = cloudi_ip_address:to_binary(ClientIpAddr),
+    SourcePort = erlang:integer_to_binary(ClientPort),
     HeadersIncoming1 = if
         SetXForwardedFor =:= true ->
-            header_set_if_not(<<"x-forwarded-for">>, PeerShort,
+            header_set_if_not(<<"x-forwarded-for">>, SourceAddress,
                               HeadersIncoming0);
         SetXForwardedFor =:= false ->
             HeadersIncoming0
     end,
     HeadersIncomingN = HeadersIncoming1#{
-                           <<"peer">> => PeerShort,
-                           <<"peer-port">> => PeerPort,
-                           <<"source-address">> => PeerLong,
-                           <<"source-port">> => PeerPort,
+                           <<"source-address">> => SourceAddress,
+                           <<"source-port">> => SourcePort,
                            <<"url-path">> => PathRaw},
     ResponseLookup = if
         WebSocketProtocol /= undefined ->
@@ -704,7 +701,13 @@ headers_external_outgoing([]) ->
     #{};
 headers_external_outgoing(ResponseInfo)
     when is_binary(ResponseInfo); is_list(ResponseInfo) ->
-    cloudi_response_info:key_value_parse(ResponseInfo).
+    HeadersOutgoing = cloudi_response_info:key_value_parse(ResponseInfo),
+    case maps:find(<<"set-cookie">>, HeadersOutgoing) of
+        {ok, Value} when is_binary(Value) ->
+            maps:put(<<"set-cookie">>, [Value], HeadersOutgoing);
+        _ ->
+            HeadersOutgoing
+    end.
 
 header_set_if_not(Key, Value, Headers) ->
     maps:update_with(Key, fun(ValueOld) ->
@@ -880,21 +883,18 @@ handle(Req0,
                             'normal'
                     end
             end,
-            PeerShort = erlang:list_to_binary(inet_parse:ntoa(ClientIpAddr)),
-            PeerLong = cloudi_ip_address:to_binary(ClientIpAddr),
-            PeerPort = erlang:integer_to_binary(ClientPort),
+            SourceAddress = cloudi_ip_address:to_binary(ClientIpAddr),
+            SourcePort = erlang:integer_to_binary(ClientPort),
             HeadersIncoming1 = if
                 SetXForwardedFor =:= true ->
-                    header_set_if_not(<<"x-forwarded-for">>, PeerShort,
+                    header_set_if_not(<<"x-forwarded-for">>, SourceAddress,
                                       HeadersIncoming0);
                 SetXForwardedFor =:= false ->
                     HeadersIncoming0
             end,
             HeadersIncomingN = HeadersIncoming1#{
-                                   <<"peer">> => PeerShort,
-                                   <<"peer-port">> => PeerPort,
-                                   <<"source-address">> => PeerLong,
-                                   <<"source-port">> => PeerPort,
+                                   <<"source-address">> => SourceAddress,
+                                   <<"source-port">> => SourcePort,
                                    <<"url-path">> => PathRaw},
             case handle_request(NameOutgoing, HeadersIncomingN,
                                 Body, Req0, State) of
