@@ -8,7 +8,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2015-2017 Michael Truog <mjtruog at protonmail dot com>
+%%% Copyright (c) 2015-2020 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -29,8 +29,8 @@
 %%% DEALINGS IN THE SOFTWARE.
 %%%
 %%% @author Michael Truog <mjtruog at protonmail dot com>
-%%% @copyright 2015-2017 Michael Truog
-%%% @version 1.7.3 {@date} {@time}
+%%% @copyright 2015-2020 Michael Truog
+%%% @version 2.0.1 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_service_monitoring_cloudi).
@@ -201,7 +201,7 @@ basic_update(ProcessInfo0) ->
 services_state(Timeout) ->
     try sys:get_state(cloudi_core_i_services_monitor, Timeout) of
         State ->
-            3 = erlang:tuple_size(State),
+            5 = erlang:tuple_size(State),
             state = erlang:element(1, State),
             {ok, erlang:element(2, State)}
     catch
@@ -214,16 +214,16 @@ services_init(undefined, ProcessInfo0, _, _, _, _) ->
 services_init(Interval, ProcessInfo0,
               MetricPrefix, UseAspectsOnly, Driver, EnvironmentLookup) ->
     {ok, Services} = services_state(Interval * 1000),
-    ets:new(?ETS_CONFIG,
-            [set, public, named_table,
-             {read_concurrency, true}]),
+    ?ETS_CONFIG = ets:new(?ETS_CONFIG,
+                          [set, public, named_table,
+                           {read_concurrency, true}]),
     true = ets:insert(?ETS_CONFIG, [{init, MetricPrefix, Driver}]),
-    ets:new(?ETS_PID2METRIC,
-            [set, public, named_table,
-             {read_concurrency, true}]),
-    ets:new(?ETS_REF2METRIC,
-            [set, public, named_table,
-             {read_concurrency, true}]),
+    ?ETS_PID2METRIC = ets:new(?ETS_PID2METRIC,
+                              [set, public, named_table,
+                               {read_concurrency, true}]),
+    ?ETS_REF2METRIC = ets:new(?ETS_REF2METRIC,
+                              [set, public, named_table,
+                               {read_concurrency, true}]),
     {InsertsN,
      ProcessInfoN} = cloudi_x_key2value:fold1(fun(_ID, Pids,
                                                   #service{} = Service, A) ->
@@ -406,7 +406,8 @@ services_update(ServicesOld, ServicesNew, ProcessInfo0, QueuedEmptySize,
             true = ets:insert(?ETS_PID2METRIC, InsertsN),
             _ = ets:select_delete(?ETS_PID2METRIC,
                                   [{{PidOld, '_', '_'},[],[true]}
-                                   || PidOld <- DeletesN])
+                                   || PidOld <- DeletesN]),
+            ok
     end,
     {services_metrics(CountInternal, CountExternal,
                       ConcurrencyInternal, ConcurrencyExternal,
@@ -560,10 +561,10 @@ service_process_metrics({ServiceMemory, ServiceMessages, ServiceReductionsNow},
              RequestPidInfo,
              InfoPidInfo,
              ProcessInfoN} = case erlang:tuple_size(State) of
-                30 -> % duo_mode == false
+                31 -> % duo_mode == false
                     state = erlang:element(1, State),
-                    RequestPid = erlang:element(23, State),
-                    InfoPid = erlang:element(24, State),
+                    RequestPid = erlang:element(24, State),
+                    InfoPid = erlang:element(25, State),
                     {RequestPidInfoValue,
                      ProcessInfo1} = if
                         RequestPid =:= undefined ->
@@ -581,20 +582,20 @@ service_process_metrics({ServiceMemory, ServiceMessages, ServiceReductionsNow},
                             process_info_update(InfoPid, ProcessInfo1)
                     end,
                     {?MAP_SIZE(erlang:element(3, State)),  % send_timeouts
-                     erlang:element(9, State),             % queued
-                     erlang:element(10, State),            % queued_size
-                     erlang:element(11, State),            % queued_word_size
-                     erlang:element(12, State),            % queued_info
+                     erlang:element(10, State),            % queued
+                     erlang:element(11, State),            % queued_size
+                     erlang:element(12, State),            % queued_word_size
+                     erlang:element(13, State),            % queued_info
                      ServiceMemory,
                      ServiceMessages,
                      ServiceReductionsNow,
                      RequestPidInfoValue,
                      InfoPidInfoValue,
                      ProcessInfo2};
-                15 -> % duo_mode == true
+                16 -> % duo_mode == true
                     state_duo = erlang:element(1, State),
-                    Dispatcher =  erlang:element(13, State),
-                    RequestPid = erlang:element(14, State),
+                    Dispatcher =  erlang:element(14, State),
+                    RequestPid = erlang:element(15, State),
                     {RequestPidInfoValue,
                      ProcessInfo1} = if
                         RequestPid =:= undefined ->
@@ -608,7 +609,7 @@ service_process_metrics({ServiceMemory, ServiceMessages, ServiceReductionsNow},
                                         ServiceReductionsNow},
                     DispatcherOutgoing = case service_state(Dispatcher) of
                         {ok, DispatcherState} -> % gen_server/proc_lib
-                            30 = erlang:tuple_size(DispatcherState),
+                            31 = erlang:tuple_size(DispatcherState),
                             state = erlang:element(1, DispatcherState),
                             ?MAP_SIZE(erlang:element(3, DispatcherState));
                         {error, _} ->
@@ -642,10 +643,10 @@ service_process_metrics({ServiceMemory, ServiceMessages, ServiceReductionsNow},
                              ProcessInfo2}
                     end,
                     {DispatcherOutgoing,
-                     erlang:element(6, State),   % queued
-                     erlang:element(7, State),   % queued_size
-                     erlang:element(8, State),   % queued_word_size
-                     erlang:element(9, State),   % queued_info
+                     erlang:element(7, State),   % queued
+                     erlang:element(8, State),   % queued_size
+                     erlang:element(9, State),   % queued_word_size
+                     erlang:element(10, State),  % queued_info
                      MemoryValue,
                      MessagesValue,
                      ReductionsNowValue,
@@ -743,12 +744,12 @@ service_process_metrics({ServiceMemory, ServiceMessages, ServiceReductionsNow},
                         QueuedEmptySize, MetricPrefix) ->
     case service_state(Pid) of
         {ok, {_, State}} -> % gen_statem
-            40 = erlang:tuple_size(State),
+            41 = erlang:tuple_size(State),
             state = erlang:element(1, State),
             Outgoing = ?MAP_SIZE(erlang:element(3, State)),  % send_timeouts
-            QueuedRequests = erlang:element(9, State),       % queued
-            QueuedRequestsSize0 = erlang:element(10, State), % queued_size
-            WordSize = erlang:element(11, State),            % queued_word_size
+            QueuedRequests = erlang:element(10, State),      % queued
+            QueuedRequestsSize0 = erlang:element(11, State), % queued_size
+            WordSize = erlang:element(12, State),            % queued_word_size
             QueuedRequestsLength = cloudi_x_pqueue4:len(QueuedRequests),
             QueuedRequestsSizeN = if
                 QueuedRequestsLength > 0, QueuedRequestsSize0 == 0 ->
