@@ -8,7 +8,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2012-2017 Michael Truog <mjtruog at protonmail dot com>
+%%% Copyright (c) 2012-2020 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -29,8 +29,8 @@
 %%% DEALINGS IN THE SOFTWARE.
 %%%
 %%% @author Michael Truog <mjtruog at protonmail dot com>
-%%% @copyright 2012-2017 Michael Truog
-%%% @version 1.7.2 {@date} {@time}
+%%% @copyright 2012-2020 Michael Truog
+%%% @version 2.0.2 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_x_quickrand).
@@ -275,7 +275,7 @@ strong_uniform(1) ->
 strong_uniform(N) when is_integer(N), N > 1 ->
     Bytes = bytes(N),
     Bits = Bytes * 8,
-    <<I:Bits/integer>> = crypto:strong_rand_bytes(Bytes),
+    <<I:Bits/unsigned-integer>> = crypto:strong_rand_bytes(Bytes),
     (I rem N) + 1.
 
 %%-------------------------------------------------------------------------
