@@ -48,7 +48,7 @@
 %%%
 %%% @author Michael Truog <mjtruog at protonmail dot com>
 %%% @copyright 2011-2020 Michael Truog
-%%% @version 1.8.0 {@date} {@time}
+%%% @version 2.0.1 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_x_uuid).
@@ -93,6 +93,7 @@
 -define(ERLANG_OTP_VERSION_18_FEATURES, true).
 -ifdef(ERLANG_OTP_VERSION_18).
 -else.
+-define(ERLANG_OTP_VERSION_19_FEATURES, true).
 -ifdef(ERLANG_OTP_VERSION_19).
 -else.
 -define(ERLANG_OTP_VERSION_20_FEATURES, true).
@@ -132,6 +133,25 @@
               timestamp_type/0,
               state/0]).
 
+-ifdef(ERLANG_OTP_VERSION_19_FEATURES).
+-type iso8601() ::
+    nonempty_list($0..$9 | $T | $- | $: | $. | $Z).
+-type uuid_string_list() ::
+    nonempty_list($0..$9 | $a..$f | $-).
+-else.
+-type iso8601() ::
+    nonempty_string().
+-type uuid_string_list() ::
+    nonempty_string().
+-endif.
+-type uuid_string_binary() ::
+    <<_:256>> | <<_:288>>.
+-type uuid_string() :: uuid_string_list() | uuid_string_binary().
+-export_type([iso8601/0,
+              uuid_string_list/0,
+              uuid_string_binary/0,
+              uuid_string/0]).
+
 -include("cloudi_x_uuid.hrl").
 
 % Erlang Binary Term Format constants
@@ -318,7 +338,8 @@ get_v1(#uuid_state{variant = ordered,
 %%-------------------------------------------------------------------------
 %% @doc
 %% ===Get the current time value in a manner consistent with the v1 UUID.===
-%% The result is an integer in microseconds.
+%% The result is an integer in microseconds since the UNIX epoch.
+%% (The UNIX epoch is 1970-01-01T00:00:00Z)
 %% @end
 %%-------------------------------------------------------------------------
 
@@ -331,7 +352,8 @@ get_v1_time() ->
 %%-------------------------------------------------------------------------
 %% @doc
 %% ===Get the current time value in a manner consistent with the v1 UUID.===
-%% The result is an integer in microseconds.
+%% The result is an integer in microseconds since the UNIX epoch.
+%% (The UNIX epoch is 1970-01-01T00:00:00Z)
 %% @end
 %%-------------------------------------------------------------------------
 
@@ -369,7 +391,7 @@ get_v1_time(Value)
           _:48>> ->
             <<TimeHigh:48, TimeLow:12>>
     end,
-    (Time - ?GREGORIAN_EPOCH_OFFSET) div 10. % microseconds since UNIX epoch
+    (Time - ?GREGORIAN_EPOCH_OFFSET) div 10.
 -else.
 get_v1_time(erlang) ->
     timestamp(erlang_now);
@@ -401,7 +423,7 @@ get_v1_time(Value)
           _:48>> ->
             <<TimeHigh:48, TimeLow:12>>
     end,
-    (Time - ?GREGORIAN_EPOCH_OFFSET) div 10. % microseconds since UNIX epoch
+    (Time - ?GREGORIAN_EPOCH_OFFSET) div 10.
 -endif.
 
 %%-------------------------------------------------------------------------
@@ -413,7 +435,7 @@ get_v1_time(Value)
 
 -spec get_v1_datetime(Value :: timestamp_type() | state() | cloudi_x_uuid() |
                                erlang:timestamp()) ->
-    string().
+    iso8601().
 
 get_v1_datetime({_, _, MicroSeconds} = Timestamp) ->
     {{DateYYYY, DateMM, DateDD},
@@ -447,7 +469,7 @@ get_v1_datetime(Value) ->
 -spec get_v1_datetime(Value :: timestamp_type() | state() | cloudi_x_uuid() |
                                erlang:timestamp(),
                       MicroSecondsOffset :: integer()) ->
-    string().
+    iso8601().
 
 get_v1_datetime(Value, MicroSecondsOffset)
     when is_integer(MicroSecondsOffset) ->
@@ -510,7 +532,7 @@ get_v3(Data) when is_binary(Data) ->
 %%-------------------------------------------------------------------------
 
 -spec get_v3(Namespace :: dns | url | oid | x500 | binary(),
-             Data :: binary() | iolist()) ->
+             Data :: iodata()) ->
     cloudi_x_uuid().
 
 get_v3(dns, Data) ->
@@ -522,13 +544,7 @@ get_v3(oid, Data) ->
 get_v3(x500, Data) ->
     get_v3(?UUID_NAMESPACE_X500, Data);
 get_v3(Namespace, Data) when is_binary(Namespace) ->
-    DataBin = if
-        is_binary(Data) ->
-            Data;
-        is_list(Data) ->
-            erlang:iolist_to_binary(Data)
-    end,
-    get_v3(<<Namespace/binary, DataBin/binary>>).
+    get_v3(erlang:iolist_to_binary([Namespace, Data])).
 
 %%-------------------------------------------------------------------------
 %% @doc
@@ -560,7 +576,7 @@ get_v3_compat(Data) when is_binary(Data) ->
 %%-------------------------------------------------------------------------
 
 -spec get_v3_compat(Namespace :: dns | url | oid | x500 | binary(),
-                    Data :: binary() | iolist()) ->
+                    Data :: iodata()) ->
     cloudi_x_uuid().
 
 get_v3_compat(dns, Data) ->
@@ -572,13 +588,7 @@ get_v3_compat(oid, Data) ->
 get_v3_compat(x500, Data) ->
     get_v3_compat(?UUID_NAMESPACE_X500, Data);
 get_v3_compat(Namespace, Data) when is_binary(Namespace) ->
-    DataBin = if
-        is_binary(Data) ->
-            Data;
-        is_list(Data) ->
-            erlang:iolist_to_binary(Data)
-    end,
-    get_v3_compat(<<Namespace/binary, DataBin/binary>>).
+    get_v3_compat(erlang:iolist_to_binary([Namespace, Data])).
 
 %%-------------------------------------------------------------------------
 %% @doc
@@ -710,7 +720,7 @@ get_v5(Data) when is_binary(Data) ->
 %%-------------------------------------------------------------------------
 
 -spec get_v5(Namespace :: dns | url | oid | x500 | binary(),
-             Data :: binary() | iolist()) ->
+             Data :: iodata()) ->
     cloudi_x_uuid().
 
 get_v5(dns, Data) ->
@@ -722,13 +732,7 @@ get_v5(oid, Data) ->
 get_v5(x500, Data) ->
     get_v5(?UUID_NAMESPACE_X500, Data);
 get_v5(Namespace, Data) when is_binary(Namespace) ->
-    DataBin = if
-        is_binary(Data) ->
-            Data;
-        is_list(Data) ->
-            erlang:iolist_to_binary(Data)
-    end,
-    get_v5(<<Namespace/binary, DataBin/binary>>).
+    get_v5(erlang:iolist_to_binary([Namespace, Data])).
 
 %%-------------------------------------------------------------------------
 %% @doc
@@ -760,7 +764,7 @@ get_v5_compat(Data) when is_binary(Data) ->
 %%-------------------------------------------------------------------------
 
 -spec get_v5_compat(Namespace :: dns | url | oid | x500 | binary(),
-                    Data :: binary() | iolist()) ->
+                    Data :: iodata()) ->
     cloudi_x_uuid().
 
 get_v5_compat(dns, Data) ->
@@ -772,13 +776,7 @@ get_v5_compat(oid, Data) ->
 get_v5_compat(x500, Data) ->
     get_v5_compat(?UUID_NAMESPACE_X500, Data);
 get_v5_compat(Namespace, Data) when is_binary(Namespace) ->
-    DataBin = if
-        is_binary(Data) ->
-            Data;
-        is_list(Data) ->
-            erlang:iolist_to_binary(Data)
-    end,
-    get_v5_compat(<<Namespace/binary, DataBin/binary>>).
+    get_v5_compat(erlang:iolist_to_binary([Namespace, Data])).
 
 %%-------------------------------------------------------------------------
 %% @doc
@@ -805,7 +803,7 @@ is_v5(_) ->
 %%-------------------------------------------------------------------------
 
 -spec uuid_to_list(cloudi_x_uuid()) ->
-    iolist().
+    nonempty_list(non_neg_integer()).
 
 uuid_to_list(<<B1:32/unsigned-integer,
                B2:16/unsigned-integer,
@@ -821,7 +819,7 @@ uuid_to_list(<<B1:32/unsigned-integer,
 %%-------------------------------------------------------------------------
 
 -spec uuid_to_string(Value :: cloudi_x_uuid()) ->
-    string().
+    uuid_string_list().
 
 uuid_to_string(Value) ->
     uuid_to_string(Value, standard).
@@ -836,7 +834,7 @@ uuid_to_string(Value) ->
                      Option :: standard | nodash |
                                list_standard | list_nodash |
                                binary_standard | binary_nodash) ->
-    string() | binary().
+    uuid_string().
 
 uuid_to_string(<<Value:128/unsigned-integer>>, standard) ->
     [N01, N02, N03, N04, N05, N06, N07, N08,
@@ -892,7 +890,7 @@ uuid_to_string(<<Value:128/unsigned-integer>>, binary_nodash) ->
 %% @end
 %%-------------------------------------------------------------------------
 
--spec string_to_uuid(string() | binary()) ->
+-spec string_to_uuid(uuid_string()) ->
     cloudi_x_uuid().
 
 string_to_uuid([N01, N02, N03, N04, N05, N06, N07, N08, $-,
@@ -1100,7 +1098,7 @@ increment(#uuid_state{variant = Variant,
 %%-------------------------------------------------------------------------
 
 -spec mac_address() ->
-    list(non_neg_integer()).
+    nonempty_list(non_neg_integer()).
 
 mac_address() ->
     {ok, Ifs} = inet:getifaddrs(),
@@ -1499,10 +1497,16 @@ pseudo_random(N) ->
 -ifdef(TEST).
 -include_lib("eunit/include/eunit.hrl").
 
-internal_test_() ->
-    [
+-ifdef(CLOUDI_TEST_TIMEOUT).
+-define(TEST_TIMEOUT, ?CLOUDI_TEST_TIMEOUT). % seconds
+-else.
+-define(TEST_TIMEOUT, 10). % seconds
+-endif.
+
+module_test_() ->
+    {timeout, ?TEST_TIMEOUT, [
         {"internal tests", ?_assertEqual(ok, test())}
-    ].
+    ]}.
 
 -endif.
 
