@@ -4,7 +4,7 @@
 #
 # MIT License
 #
-# Copyright (c) 2017 Michael Truog <mjtruog at gmail dot com>
+# Copyright (c) 2017-2021 Michael Truog <mjtruog at protonmail dot com>
 #
 # Permission is hereby granted, free of charge, to any person obtaining a
 # copy of this software and associated documentation files (the "Software"),
@@ -35,12 +35,14 @@ if __FILE__ == $PROGRAM_NAME
     threads = (0...thread_count).to_a.map{ |i| Thread.new(i){ |thread_index|
         class Task
             def initialize(thread_index)
-                @api = CloudI::API.new(thread_index)
+                @api = nil
+                @thread_index = thread_index
                 @count = 0
             end
 
             def run
                 begin
+                    @api = CloudI::API.new(@thread_index)
                     @api.subscribe('ruby/get', method(:request))
 
                     result = @api.poll
@@ -69,17 +71,13 @@ if __FILE__ == $PROGRAM_NAME
                 end
                 $stdout.puts "count == #{@count} ruby"
                 response = "#{@count}"
+                response_info = CloudI::API.info_key_value_new({})
                 @api.return_(request_type, name, pattern,
-                             '', response, timeout, trans_id, pid)
+                             response_info, response, timeout, trans_id, pid)
             end
         end
-        begin
-            object = Task.new(thread_index)
-            object.run
-        rescue
-            $stderr.puts $!.message
-            $stderr.puts $!.backtrace
-        end
+
+        Task.new(thread_index).run
     }}
     threads.each{ |t| t.join }
 end
