@@ -4,7 +4,7 @@
 //
 // MIT License
 //
-// Copyright (c) 2014-2017 Michael Truog <mjtruog at gmail dot com>
+// Copyright (c) 2014-2021 Michael Truog <mjtruog at protonmail dot com>
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -30,16 +30,19 @@ require 'CloudI.php';
 class Task //extends \Thread
 {
     private $api;
+    private $thread_index;
 
-    public function __construct($api)
+    public function __construct($thread_index)
     {
-        $this->api = $api;
+        $this->api = null;
+        $this->thread_index = $thread_index;
     }
 
     public function run()
     {
         try
         {
+            $this->api = new \CloudI\API($this->thread_index);
             assert($this->api->subscribe_count('php.xml/get') == 0);
             $this->api->subscribe('php.xml/get', $this, 'request');
             assert($this->api->subscribe_count('php.xml/get') == 1);
@@ -60,7 +63,7 @@ class Task //extends \Thread
                             $request_info, $request,
                             $timeout, $priority, $trans_id, $pid)
     {
-        $http_qs = $this->api->info_key_value_parse($request);
+        $http_qs = \CloudI\API::info_key_value_parse($request);
         if (! isset($http_qs['value']))
         {
             $response =
@@ -75,15 +78,18 @@ class Task //extends \Thread
             $response =
 "<http_test><value>{$value}</value></http_test>";
         }
+        $response_info = \CloudI\API::info_key_value_new(array(
+            'content-type' => 'text/xml; charset=utf-8',
+        ));
         $this->api->return_($request_type, $name, $pattern,
-                            '', $response,
+                            $response_info, $response,
                             $timeout, $trans_id, $pid);
     }
 }
 
 $thread_count = \CloudI\API::thread_count();
 assert($thread_count == 1);
-$main_thread = new Task(new \CloudI\API(0));
+$main_thread = new Task(0);
 $main_thread->run();
 
 /*
@@ -91,11 +97,11 @@ $main_thread->run();
 // readily available installation packages
 $thread_count = \CloudI\API::thread_count();
 assert($thread_count >= 1);
-    
+
 $threads = array();
-for ($i = 0; $i < $thread_count; $i++)
+for ($thread_index = 0; $thread_index < $thread_count; $thread_index++)
 {
-    $threads[$i] = new Task(new \CloudI\API($i));
+    $threads[$thread_index] = new Task($thread_index);
 }
 foreach ($threads as $t)
     $t->start();
