@@ -4,7 +4,7 @@
 #
 # MIT License
 #
-# Copyright (c) 2012-2020 Michael Truog <mjtruog at protonmail dot com>
+# Copyright (c) 2012-2021 Michael Truog <mjtruog at protonmail dot com>
 #
 # Permission is hereby granted, free of charge, to any person obtaining a
 # copy of this software and associated documentation files (the "Software"),
@@ -35,12 +35,13 @@ if __FILE__ == $PROGRAM_NAME
     threads = (0...thread_count).to_a.map{ |i| Thread.new(i){ |thread_index|
         class Task
             def initialize(thread_index)
-                @api = CloudI::API.new(thread_index)
+                @api = nil
                 @thread_index = thread_index
             end
 
             def run
                 begin
+                    @api = CloudI::API.new(@thread_index)
                     @api.subscribe('a/b/c/d', method(:sequence1_abcd))
                     @api.subscribe('a/b/c/*', method(:sequence1_abc_))
                     @api.subscribe('a/b/*/d', method(:sequence1_ab_d))
@@ -445,13 +446,8 @@ if __FILE__ == $PROGRAM_NAME
                              '', 'end', timeout, trans_id, pid)
             end
         end
-        begin
-            object = Task.new(thread_index)
-            object.run
-        rescue
-            $stderr.puts $!.message
-            $stderr.puts $!.backtrace
-        end
+
+        Task.new(thread_index).run
     }}
     threads.each{ |t| t.join }
 end
